//////////////////////////////////////////////
// lexical analyser
//
//    FILE : analyser.cpp
// CREATED : 11.07.2000 
//
// (c) Borisov E.S. Kiev UA
//
//       OS : QNX 4.25
// COMPILER : Watcom C/C++ 10.6
///////////////////////////////////////////////
#include <iostream.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "analyser.h"

//=========================================================
// FUNCTION :CAnalyser()
// PURPOSE  : 
//
CAnalyser::CAnalyser():m_textsize(0),m_whead(0)
{
   sprintf(m_text,"");
}

//=========================================================
// FUNCTION :~CAnalyser()
// PURPOSE  : 
//
CAnalyser::~CAnalyser(){ del_gram(); }

//=========================================================
// FUNCTION : void set_text(char* text,int size);
// PURPOSE  : set text buffer
//
void CAnalyser::set_text(char* text,int size)
{
   m_textsize=min(size,MAX_BUF);
   memcpy(m_text,text,m_textsize);
}

//=========================================================
// FUNCTION : int load_text(...) 
// PURPOSE  : load text from file
//
int CAnalyser::load_text(char* szdatafile) 
{
    int fd=-1;
    //////////////////////////////////////////
    // open file
    if( (fd = open( szdatafile, 
    	            O_RDONLY,
    	            S_IRUSR | S_IWUSR) )==-1 )
    {
        cerr<<"analyser open '"<<szdatafile<<"' - "<<strerror(errno)<<endl;
        return 2;
    }
    //
    //////////////////////////////////////////
    
    /////////////////////////////////////////////////
    // read in buffer
    if((m_textsize=read(fd, m_text,sizeof(m_text)))<1 )
    {
         cerr<<"analyser read - "<<strerror(errno)<<endl;
         close(fd);
         return 3;
    }
    //
    /////////////////////////////////////////////////
    close(fd);
    return 0;
}

//=========================================================
// FUNCTION : int add_lex(char* l)
// PURPOSE  : add lexeme in stack
//
int CAnalyser::add_lex(char* l)
{
   t_lstack* pbuf= new t_lstack;
   
   if(pbuf==0)
   {
      cerr<<"analyser new - "<<strerror(errno)<<endl;
      return 1;
   }
   sprintf(pbuf->lex,"%s",l);
   pbuf->link=m_whead;
   m_whead=pbuf;  
   
   return 0;
} 

//=========================================================
// FUNCTION : void del_gram()
// PURPOSE  : delete lexeme stack
//
void CAnalyser::del_gram()
{
   t_lstack*  wptr;

   while(wptr=m_whead) 
   {
     m_whead=m_whead->link;
     delete wptr;
   }
} 

//=========================================================
// FUNCTION : void print_gram()
// PURPOSE  : print lexeme stack
//
void CAnalyser::print_gram()
{
   t_lstack*  wptr;

   cout<<"LEX:\n--------\n";
   if(!(wptr=m_whead))return;
   do
   {
      cout<<wptr->lex<<endl;
         
   }while(wptr=wptr->link);
   cout<<"-----\n";
} 

//=========================================================
// FUNCTION : int load_gram() 
// PURPOSE  : load lexeme stack from file
//
int CAnalyser::load_gram(char* szlexfile) 
{
    int fd=-1;
    char cbuf=0;
    char szbuf[MAX_BUF]="";
    

    //////////////////////////////////////////
    // open file
    if( (fd = open( szlexfile, 
    	            O_RDONLY,
    	            S_IRUSR | S_IWUSR) )==-1 )
    {
        cerr<<"wordlist open '"<<szlexfile<<"' - "<<strerror(errno)<<endl;
        return 2;
    }
    //
    //////////////////////////////////////////
    
    if(m_whead)del_gram();

    while(!eof(fd))    
    {
        
        /////////////////////////////////////////////////
        // read in buffer
        if(read(fd, &cbuf,sizeof(cbuf))<1 )
        {
           cerr<<"wordlist read  '"<<szlexfile<<"' - "<<strerror(errno)<<endl;
           close(fd);
           return 3;
        }
        //
        /////////////////////////////////////////////////

        ////////////////////////////////////////////
        // fill buffer
        if(isalpha(cbuf))
        {
           sprintf(szbuf,"%s%c",szbuf,cbuf);
        }
        else
        {
           if(strlen(szbuf))
           {
              if(add_lex(szbuf))
              {
                 close(fd);
                 return 4;
              }
           }
           sprintf(szbuf,"");
        }
        //     
        ////////////////////////////////////////////
   }
   close(fd);

   return 0;
}

//=========================================================
// FUNCTION : int get_lex_quantity()
// PURPOSE  : 
//
int CAnalyser::get_lex_quantity() const
{
   t_lstack*  wptr;
   int lcount=0;

   if(!(wptr=m_whead))return 0;
   do{ lcount++; }while(wptr=wptr->link);
   return lcount;
}

//=========================================================
// FUNCTION : int strlen(char* str)
// PURPOSE  : 
//
int inline CAnalyser::strlen(char* str)
{
   if(str==0)return 0;
   return ::strlen(str);
}

//=========================================================
// FUNCTION : int find_word(char* str,char* word)
// PURPOSE  : find word in string 
//
int CAnalyser::find_word(char* str,char* word)
{
    int   pos=0;
    int   wcount=0;    
    
    if( !(strlen(str)&&strlen(word)) )return 0;
    if(pos=strlen(strstr(str,word)) )
    {
       wcount++;
       pos=m_textsize-pos+strlen(word)+1;
       wcount+=find_word(&str[pos],word);
    }
    return wcount;
}

//=========================================================
// FUNCTION : int analyse()
// PURPOSE  : analysing text
//
int CAnalyser::analyse()
{
   t_lstack*  wptr;
   int wcount=0;

   if(!(wptr=m_whead))return 0;
   
   do
   {
	  wcount+=find_word(m_text,wptr->lex);
   }while(wptr=wptr->link);

   return wcount;
}